@@ -36,8 +36,10 @@
 #include "gdkmacosglcontext-private.h"
 #include "gdkmacoskeymap-private.h"
 #include "gdkmacosmonitor-private.h"
+#include "gdkmacospopupsurface-private.h"
 #include "gdkmacosseat-private.h"
 #include "gdkmacossurface-private.h"
+#include "gdkmacostoplevelsurface-private.h"
 #include "gdkmacosutils-private.h"
 
 G_DEFINE_TYPE (GdkMacosDisplay, gdk_macos_display, GDK_TYPE_DISPLAY)
@@ -156,9 +158,8 @@ gdk_macos_display_update_bounds (GdkMacosDisplay *self)
   self->width = self->max_x - self->min_x;
   self->height = self->max_y - self->min_y;
 
-  GDK_NOTE (MISC,
-            g_message ("Displays reconfigured to bounds %d,%d %dx%d",
-                       self->min_x, self->min_y, self->width, self->height));
+  GDK_DEBUG (MISC, "Displays reconfigured to bounds %d,%d %dx%d",
+                   self->min_x, self->min_y, self->width, self->height);
 
   GDK_END_MACOS_ALLOC_POOL;
 }
@@ -284,7 +285,7 @@ static void
 push_nsevent (GdkEvent *gdk_event,
               NSEvent  *nsevent)
 {
-  GdkToNSEventMap *map = g_slice_new0 (GdkToNSEventMap);
+  GdkToNSEventMap *map = g_new0 (GdkToNSEventMap, 1);
 
   map->link.data = map;
   map->gdk_event = gdk_event_ref (gdk_event);
@@ -298,7 +299,7 @@ push_nsevent (GdkEvent *gdk_event,
 
       gdk_event_unref (map->gdk_event);
       [map->nsevent release];
-      g_slice_free (GdkToNSEventMap, map);
+      g_free (map);
     }
 }
 
@@ -334,7 +335,7 @@ gdk_macos_display_queue_events (GdkDisplay *display)
     }
 }
 
-static void
+void
 _gdk_macos_display_surface_added (GdkMacosDisplay *self,
                                   GdkMacosSurface *surface)
 {
@@ -434,7 +435,8 @@ select_key_in_idle_cb (gpointer data)
         {
           GdkMacosSurface *surface = iter->data;
 
-          if (GDK_SURFACE_IS_MAPPED (GDK_SURFACE (surface)))
+          if (GDK_SURFACE_IS_MAPPED (GDK_SURFACE (surface)) &&
+              ([surface->window styleMask] & NSWindowStyleMaskMiniaturizable) == 0)
             {
               [surface->window showAndMakeKey:YES];
               break;
@@ -539,29 +541,6 @@ _gdk_macos_display_surface_resigned_main (GdkMacosDisplay *self,
   _gdk_macos_display_clear_sorting (self);
 }
 
-static GdkSurface *
-gdk_macos_display_create_surface (GdkDisplay     *display,
-                                  GdkSurfaceType  surface_type,
-                                  GdkSurface     *parent,
-                                  int             x,
-                                  int             y,
-                                  int             width,
-                                  int             height)
-{
-  GdkMacosDisplay *self = (GdkMacosDisplay *)display;
-  GdkMacosSurface *surface;
-
-  g_assert (GDK_IS_MACOS_DISPLAY (self));
-  g_assert (!parent || GDK_IS_MACOS_SURFACE (parent));
-
-  surface = _gdk_macos_surface_new (self, surface_type, parent, x, y, width, height);
-
-  if (surface != NULL)
-    _gdk_macos_display_surface_added (self, surface);
-
-  return GDK_SURFACE (surface);
-}
-
 static GdkKeymap *
 gdk_macos_display_get_keymap (GdkDisplay *display)
 {
@@ -617,10 +596,11 @@ gdk_macos_display_class_init (GdkMacosDisplayClass *klass)
 
   object_class->finalize = gdk_macos_display_finalize;
 
+  display_class->toplevel_type = GDK_TYPE_MACOS_TOPLEVEL_SURFACE;
+  display_class->popup_type = GDK_TYPE_MACOS_POPUP_SURFACE;
   display_class->cairo_context_type = GDK_TYPE_MACOS_CAIRO_CONTEXT;
 
   display_class->beep = gdk_macos_display_beep;
-  display_class->create_surface = gdk_macos_display_create_surface;
   display_class->flush = gdk_macos_display_flush;
   display_class->get_keymap = gdk_macos_display_get_keymap;
   display_class->get_monitors = gdk_macos_display_get_monitors;
@@ -662,7 +642,7 @@ _gdk_macos_display_open (const char *display_name)
     return NULL;
 
   display_name = display_name ? display_name : "";
-  GDK_NOTE (MISC, g_message ("opening display %s", display_name));
+  GDK_DEBUG (MISC, "opening display %s", display_name);
 
   /* Make the current process a foreground application, i.e. an app
    * with a user interface, in case we're not running from a .app bundle
@@ -870,15 +850,12 @@ _gdk_macos_display_get_surface_at_coords (GdkMacosDisplay *self,
   for (const GList *iter = surfaces; iter; iter = iter->next)
     {
       GdkSurface *surface = iter->data;
-      NSWindow *nswindow;
 
       g_assert (GDK_IS_MACOS_SURFACE (surface));
 
       if (!gdk_surface_get_mapped (surface))
         continue;
 
-      nswindow = _gdk_macos_surface_get_native (GDK_MACOS_SURFACE (surface));
-
       if (x >= GDK_MACOS_SURFACE (surface)->root_x &&
           y >= GDK_MACOS_SURFACE (surface)->root_y &&
           x <= (GDK_MACOS_SURFACE (surface)->root_x + surface->width) &&
@@ -1027,6 +1004,16 @@ _gdk_macos_display_get_nsevent (GdkEvent *event)
   return NULL;
 }
 
+NSEvent *
+_gdk_macos_display_get_last_nsevent ()
+{
+  const GdkToNSEventMap *map = g_queue_peek_tail (&event_map);
+  if (map)
+    return map->nsevent;
+
+  return NULL;
+}
+
 GdkDrag *
 _gdk_macos_display_find_drag (GdkMacosDisplay *self,
                               NSInteger        sequence_number)
